# Iran 2009 ballot boxes

asc <- function(x) { as.character(x) }

cvec <- c(
  "xls2csv ../ballots/Qazvin.xls");

pnames <- gsub("\\.xls[[:print:][:cntrl:]]*$","",gsub("xls2csv \\.\\./ballots/","",cvec));
keepv <- list();
keepv[[1]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No","Township","County");
keepv[[2]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No","County","Township");
keepv[[3]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No");

datlist <- list();
for (j in 1:(length(cvec))) {
  cmd <- cvec[j];
  if (pnames[j] %in% c("kordestan","yazd")) {
    dat <- read.csv(pipe(cmd));
    vkeep <- keepv[[1]];
  }
  else if (pnames[j] %in% c("Markazi-fix","NorthKhorasan","eelam")) {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=c("X1",keepv[[2]]));
    vkeep <- keepv[[2]];
  }
  else {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=keepv[[3]]);
    vkeep <- keepv[[3]];
  }
  print(pnames[j]);
  print(dim(dat));
  print(names(dat));
  datlist[[pnames[j]]] <- dat[,vkeep];
}
pnames0 <- pnames;

csvfiles <- system("ls -1 *.csv | grep -v boxes-version1.csv", intern=TRUE);
pnames <- gsub("\\.csv$","",csvfiles);

h <- c("Total", "Void", "Correct", "Mousavi", "Karoubi", "Rezaei",
"Ahmadinejad", "UnitNo", "Township", "Province", "Province2");

for (j in 1:(length(csvfiles))) {
  cmd <- paste("awk 'NR > 1 && $0 !~ /[tT]otal/ && $0 !~ /,,,,,,,,,/' '",csvfiles[j],"'",sep="");
  if (csvfiles[j] %in% c("semnan.csv","Guilan.csv")) {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=c(h,paste("X",1:3,sep="")));
  }
  else {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=h);
  }
  print(csvfiles[j]);
  print(dim(dat));
  print(names(dat));
  datlist[[pnames[j]]] <- dat[,h];
}
pnames1 <- pnames;

getdigits <- function(num,digit) {
  s <- as.character(num);
  idx <- sapply(s, function(x){ nchar(x) >= digit; });
  dout <- ifelse(idx, NA, "");
  if (any(idx)) {
    dout[idx] <- sapply(s[idx], function(x){ substr(x,digit,digit) });
  }
  return(dout);
}

# Benford's Law probablity formulas
# source
# http://www.mathpages.com/home/kmath302/kmath302.htm
#
# PndB:  d, digit value (vector);  n, digit place;  B, base
# probability that the n-th digit following the first nonzero digit is d,
#   for numbers in base B
# d can be a vector;  n and B must be positive scalars
#
PndB <- function(d,n,B=10) {
  if (n==0) {
    p <- log(1+1/d)/log(B);
  }
  else {
    seq <- B^(n-1):(B^n-1);
    p <- d*0;
    for (i in 1:length(d)) {
      p[i] <- sum(log(1+1/(d[i]+seq*B)))/log(B);
    }
  }
  return(p);
}

runtest <- function(wrk, digit=2) {
  least <- ifelse(digit==1, 1, 0);  # initial digit for Benford test
  vec <- least:9;
  set <- rep(0,length(vec));
  names(set) <- as.character(vec);
  ndtable <- names(dtable <- table(getdigits(wrk, digit=digit)));
  set[ndtable[ndtable %in% vec]] <- dtable[ndtable[ndtable %in% vec]];
  sumset <- sum(set);
  # Benford's law logarithmic distribution test
  bpred <- sumset*PndB(vec, digit-1);
  chi <- sum(dev <- (set - bpred)^2 / bpred);
  dev <- sign(set - bpred) * sqrt(dev);
  # uniform distribution test
  upred <- rep(ifelse(digit==1,1/9,.1), ifelse(digit==1,9,10))*sumset;
  chiU <- sum((set - upred)^2 / upred);
#  if(is.na(chi)) print( table(getdigits(wrk, digit=digit)) );
  return(list(dtable=dtable, sumset=sumset, chiU=chiU, chi=chi, dev=dev));
}

pnames <- unique(c(pnames0,pnames1));
pnames;
vnames <- c("Mousavi","Karoubi","Rezaei","Ahmadinejad");

marr <- array(NA, dim=c(length(vnames), 2, 1+length(pnames)));
dimnames(marr) <- list(vnames, c("sumset","chiB"), c("all",pnames));

nall <- 0;
for (k in pnames) nall <- nall + dim(datlist[[k]])[1];
allmat <- matrix(NA,nall,length(vnames));
dimnames(allmat)[[2]] <- vnames;
ii <- 0;
for (k in pnames) {
  kdim <- dim(datlist[[k]]);
  kmat <- array(as.numeric(as.matrix(datlist[[k]][,vnames])),dim=c(kdim[1],length(vnames)));
  if (any(is.na(kmat))) {
    print(k);
    kidx <- apply(is.na(kmat),1,any);
    print((1:dim(kmat)[1])[kidx]);
    print(kmat[kidx,]);
    if (k == "ilam") {
      kmat <- kmat[!kidx,];
    }
  }
  dimnames(kmat)[[2]] <- vnames;
  for (j in 1:length(vnames)) {
    tt <- runtest(kmat[,vnames[j]]);
    marr[vnames[j],,k] <- c(tt$sumset, tt$chi);
  }
  allmat[ii+(1:dim(kmat)[1]),] <- kmat;
  ii <- ii + kdim[1];
}
idx <- apply(is.na(allmat),1,any);
(1:dim(allmat)[1])[idx];
allmat[idx,];
for (j in 1:length(vnames)) {
  tt <- runtest(allmat[,vnames[j]]);
  marr[vnames[j],,"all"] <- c(tt$sumset, tt$chi);
}
marr;

qchisq(1-.01/4,9);
qchisq(1-.05/4,9);
qchisq(1-.1/4,9);
qchisq(1-.2/4,9);

runtest(c(allmat[,vnames[1:4]]));
qchisq(1-.01,9);
qchisq(1-.05,9);
qchisq(1-.1,9);
qchisq(1-.2,9);
