# Iran 2009 ballot boxes

library(sm);

asc <- function(x) { as.character(x) }

cvec <- c(
  "xls2csv ../ballots/Qazvin.xls");

pnames <- gsub("\\.xls[[:print:][:cntrl:]]*$","",gsub("xls2csv \\.\\./ballots/","",cvec));
keepv <- list();
keepv[[1]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No","Township","County");
keepv[[2]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No","County","Township");
keepv[[3]] <- c("Total","Void","Correct","Mousavi","Karoubi","Rezaei","Ahmadinejad",
 "Unit..No");

datlist <- list();
for (j in 1:(length(cvec))) {
  cmd <- cvec[j];
  if (pnames[j] %in% c("kordestan","yazd")) {
    dat <- read.csv(pipe(cmd));
    vkeep <- keepv[[1]];
  }
  else if (pnames[j] %in% c("Markazi-fix","NorthKhorasan","eelam")) {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=c("X1",keepv[[2]]));
    vkeep <- keepv[[2]];
  }
  else {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=keepv[[3]]);
    vkeep <- keepv[[3]];
  }
  print(pnames[j]);
  print(dim(dat));
  print(names(dat));
  datlist[[pnames[j]]] <- dat[,vkeep];
}
pnames0 <- pnames;

csvfiles <- system("ls -1 *.csv | grep -v boxes-version1.csv", intern=TRUE);
pnames <- gsub("\\.csv$","",csvfiles);

h <- c("Total", "Void", "Correct", "Mousavi", "Karoubi", "Rezaei",
"Ahmadinejad", "UnitNo", "Township", "Province", "Province2");

for (j in 1:(length(csvfiles))) {
  cmd <- paste("awk 'NR > 1 && $0 !~ /[tT]otal/ && $0 !~ /,,,,,,,,,/' '",csvfiles[j],"'",sep="");
  if (csvfiles[j] %in% c("semnan.csv","Guilan.csv")) {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=c(h,paste("X",1:3,sep="")));
  }
  else {
    dat <- read.csv(pipe(cmd), header=FALSE, col.names=h);
  }
  print(csvfiles[j]);
  print(dim(dat));
  print(names(dat));
  datlist[[pnames[j]]] <- dat[,h];
}
pnames1 <- pnames;

getdigits <- function(num,digit) {
  s <- as.character(num);
  idx <- sapply(s, function(x){ nchar(x) >= digit; });
  dout <- ifelse(idx, NA, "");
  if (any(idx)) {
    dout[idx] <- sapply(s[idx], function(x){ substr(x,digit,digit) });
  }
  return(dout);
}

# Benford's Law probablity formulas
# source
# http://www.mathpages.com/home/kmath302/kmath302.htm
#
# PndB:  d, digit value (vector);  n, digit place;  B, base
# probability that the n-th digit following the first nonzero digit is d,
#   for numbers in base B
# d can be a vector;  n and B must be positive scalars
#
PndB <- function(d,n,B=10) {
  if (n==0) {
    p <- log(1+1/d)/log(B);
  }
  else {
    seq <- B^(n-1):(B^n-1);
    p <- d*0;
    for (i in 1:length(d)) {
      p[i] <- sum(log(1+1/(d[i]+seq*B)))/log(B);
    }
  }
  return(p);
}

# 2d digit Benford frequencies
benf2 <- PndB(0:9,1);
names(benf2) <- as.character(0:9);

# 2d digit Benford mean
print(benf2mean <- c(benf2 %*% 0:9));

runtest <- function(wrk, digit=2) {
  least <- ifelse(digit==1, 1, 0);  # initial digit for Benford test
  vec <- least:9;
  set <- rep(0,length(vec));
  names(set) <- as.character(vec);
  ndtable <- names(dtable <- table(getdigits(wrk, digit=digit)));
  set[ndtable[ndtable %in% vec]] <- dtable[ndtable[ndtable %in% vec]];
  sumset <- sum(set);
  # Benford's law logarithmic distribution test
  bpred <- sumset*PndB(vec, digit-1);
  chi <- sum(dev <- (set - bpred)^2 / bpred);
  dev <- sign(set - bpred) * sqrt(dev);
  # uniform distribution test
  upred <- rep(ifelse(digit==1,1/9,.1), ifelse(digit==1,9,10))*sumset;
  chiU <- sum((set - upred)^2 / upred);
#  if(is.na(chi)) print( table(getdigits(wrk, digit=digit)) );
  return(list(dtable=dtable, sumset=sumset, chiU=chiU, chi=chi, dev=dev));
}

pnames <- unique(c(pnames0,pnames1));
pnames;
vnames <- c("Void","Total","Mousavi","Karoubi","Rezaei","Ahmadinejad");

nall <- 0;
for (k in pnames) nall <- nall + dim(datlist[[k]])[1];
allmat <- matrix(NA,nall,length(vnames));
dimnames(allmat)[[2]] <- vnames;
ii <- 0;
for (k in pnames) {
  kdim <- dim(datlist[[k]]);
  kmat <- array(as.numeric(as.matrix(datlist[[k]][,vnames])),dim=c(kdim[1],length(vnames)));
  if (any(is.na(kmat))) {
    print(k);
    kidx <- apply(is.na(kmat),1,any);
    print((1:dim(kmat)[1])[kidx]);
    print(kmat[kidx,]);
    if (k == "ilam") {
      kmat <- kmat[!kidx,];
    }
  }
  dimnames(kmat)[[2]] <- vnames;
  allmat[ii+(1:dim(kmat)[1]),] <- kmat;
  ii <- ii + kdim[1];
}

print(tablast <- table(allmat[,"Total"] %% 10));
print(plast <- tablast/sum(tablast));
print(devlast <- sign((tablast-0.1*sum(tablast)))*
  sqrt((tablast-0.1*sum(tablast))^2/(0.1*sum(tablast))));
sum(devlast^2);

print(tablast2 <- table(allmat[,"Total"] %% 100));
print(plast2 <- tablast2/sum(tablast2));
print(devlast2 <- sign((tablast2-0.01*
  sum(tablast2)))*sqrt((tablast2-0.01*sum(tablast2))^2/(0.01*sum(tablast2))));
sum(devlast2^2);

print(tablast50 <- table(allmat[,"Total"] %% 50));
print(plast50 <- tablast50/sum(tablast50));
print(devlast50 <- sign((tablast50-0.02*
  sum(tablast50)))*sqrt((tablast50-0.02*sum(tablast50))^2/(0.02*sum(tablast50))));
sum(devlast50^2);

all50 <- (T50 <- allmat[,"Total"] %% 50) == 0;
alltotal <- apply(allmat,1,sum,na.rm=TRUE);
allinvalid <- allmat[,"Void"]/alltotal;

pmat <- dmat <- matrix(NA,nall,length(vnames));
dimnames(pmat)[[2]] <- dimnames(dmat)[[2]] <- vnames;
for (j in vnames) {
  pmat[,j] <- allmat[,j]/alltotal;
  dmat[,j] <- as.numeric(getdigits(allmat[,j], digit=2));
}

logit <- function(x) { z <- log(x/(1-x)); ifelse(is.finite(z),z,NA); }

summary(lm(Void ~ all50, as.data.frame(pmat)));
summary(lmstart <- lm(I(logit(pmat[,"Void"])) ~ all50));

library(multinomRob);

tdat <- data.frame(allmat);
tdat$obs <- 1:dim(allmat)[1];
tdat$NotVoid <- tdat$Total - tdat$Void;
tdat$Tot50 <- ifelse((tdat$Total %% 50) == 0,1,0);
tidx <- apply(as.matrix(tdat[,c("Void","NotVoid","Tot50")]),1,function(x){ all(!is.na(x)) });
tdat[!tidx,];
tdat <- tdat[tidx,];
summary(glm50 <- glm(cbind(Void,NotVoid) ~ Tot50, tdat, family=quasibinomial));
#m50 <- multinomRob(list(Void ~ Tot50, NotVoid ~ 0), tdat, starting.values=glm50$coef, print=3);

smat <- matrix(NA,sum(tdat$Tot50)*3,4);
len50 <- sum(tdat$Tot50);
kvars <- c("Void","NotVoid","Tot50","obs");
smat[1:len50,] <- as.matrix(tdat[tdat$Tot50,kvars]);
smat[-(1:len50),] <- 
  as.matrix(tdat[!tdat$Tot50,kvars][sample(sum(!tdat$Tot50),2*len50),]);
sdat <- data.frame(smat);
names(sdat) <- kvars;
#m50 <- multinomRob(list(Void ~ Tot50, NotVoid ~ 0), sdat, starting.values=glm50$coef, print=3,
#  genoud.parms=list(max.generations=2,hard.generation.limit=TRUE));

qlevel <- qnorm(1-(.05/2)/4);  #  Bonferroni-adjusted test level

postscript(file=paste("mod50.ps",sep=""),
  onefile=TRUE, horizontal=FALSE, width=6, height=6);

par(mfrow=c(2,2));
limvec <- Inf;
iidx <- T50 < limvec & !all50;
iidx[is.na(iidx)] <- FALSE;
Mrange <- range(T50[iidx],na.rm=TRUE);
Mrange[2] <-
  ifelse(Mrange[2]<=limvec,Mrange[2], max(T50[iidx & T50<=limvec],na.rm=TRUE));
for (j in 3:length(vnames)) {
  smr <- sm.regression(T50[iidx], dmat[iidx,vnames[j]], display="none",
    eval.points=T50);
  nlabel <- paste("n =",sum(smr$data$freq),"ballot boxes",sep=" ");
  Eidx <- is.finite(smr$estimate) & smr$eval <= Mrange[2];
  Erange <-
    range(c((smr$estimate+smr$se*qlevel)[Eidx],
      (smr$estimate-smr$se*qlevel)[Eidx]),na.rm=TRUE);
#  plot(Mrange,c(max(0,Erange[1]),Erange[2]),type="n", main=paste(j,vnames[j],sep=" "),
  plot(Mrange,Erange,type="n", main=vnames[j],
    ylab="Vote Proportion",
    xlab=paste("Proportion Invalid",nlabel,sep="\n"));
  lines(smr$eval.points[Eidx], smr$estimate[Eidx], pch=".");
#  points(smr$eval.points[Eidx], smr$estimate[Eidx], pch=".", cex=2.5);
  points(smr$eval.points[Eidx], (smr$estimate+smr$se*qlevel)[Eidx], pch=".", cex=1.5);
  points(smr$eval.points[Eidx], (smr$estimate-smr$se*qlevel)[Eidx], pch=".", cex=1.5);
  lines(Mrange,c(benf2mean,benf2mean),lty=3);
#  rug(T50[T50<=limvec]);
}
dev.off();
