# Iran 2005  and 2009 district vote counts

asc <- function(x) { as.character(x) }

asnc <- function(x) { as.numeric(as.character(x)) }

h05 <- c(
"Ahmadinejad.A.pct","Ahmadinejad.A","Rafsanjani.A.pct","Rafsanjani.A",
"unk.A","total.A","invalid.A","ballots.A","unknown.A",
"grand.total",
"Rafsanjani.B.pct","Rafsanjani.B","Ahmadinejad.B.pct","Ahmadinejad.B",
"unk.B","total.B","invalid.B","ballots.B","unknown.B",
"town","province");
dat05 <- read.delim(pipe("awk 'NR>2 {print $0;}' branch.htm.tab"), head=FALSE, col.names=h05);
dat05$total05 <-
  apply(as.matrix(dat05[,c("Ahmadinejad.A","Rafsanjani.A")]),1,sum);
dim(dat05);
names(dat05);
#> dim(dat05);
#[1] 325  22
#> names(dat05);
# [1] "Ahmadinejad.A.pct" "Ahmadinejad.A"     "Rafsanjani.A.pct" 
# [4] "Rafsanjani.A"      "unk.A"             "total.A"          
# [7] "invalid.A"         "ballots.A"         "unknown.A"        
#[10] "grand.total"       "Rafsanjani.B.pct"  "Rafsanjani.B"     
#[13] "Ahmadinejad.B.pct" "Ahmadinejad.B"     "unk.B"            
#[16] "total.B"           "invalid.B"         "ballots.B"        
#[19] "unknown.B"         "town"              "province"         
#[22] "total05"          

sapply(c(1:19,22),function(j){ sum(dat05[,j]) });

dat05R1 <- read.csv("shora-count.csv");
dim(dat05R1);
names(dat05R1);
#> dim(dat05R1);
#[1] 326   7
#> names(dat05R1);
#[1] "mehr_alizadeh" "larijani"      "ghalibaf"      "moeen"        
#[5] "karobi"        "ahmadi_nejad"  "rafsanjani"   

dat <- cbind(dat05, dat05R1[-326,]);
names(dat);
#> names(dat);
# [1] "Ahmadinejad.A.pct" "Ahmadinejad.A"     "Rafsanjani.A.pct" 
# [4] "Rafsanjani.A"      "unk.A"             "total.A"          
# [7] "invalid.A"         "ballots.A"         "unknown.A"        
#[10] "grand.total"       "Rafsanjani.B.pct"  "Rafsanjani.B"     
#[13] "Ahmadinejad.B.pct" "Ahmadinejad.B"     "unk.B"            
#[16] "total.B"           "invalid.B"         "ballots.B"        
#[19] "unknown.B"         "town"              "province"         
#[22] "total05"           "mehr_alizadeh"     "larijani"         
#[25] "ghalibaf"          "moeen"             "karobi"           
#[28] "ahmadi_nejad"      "rafsanjani"       

logistic <- function(x) { 1/(1+exp(-x)) }
logit <- function(x) { log(x/(1-x)) }

library(multinomRob);

# cross-year models

dat$total05R1 <- 
apply(as.matrix(dat[,
  c("mehr_alizadeh","larijani","ghalibaf","moeen","karobi","ahmadi_nejad","rafsanjani")]),1,sum);
dat[,
  c("mehr_alizadeh","larijani","ghalibaf","moeen","karobi","ahmadi_nejad","rafsanjani")] <-
dat[,
  c("mehr_alizadeh","larijani","ghalibaf","moeen","karobi","ahmadi_nejad","rafsanjani")]/
  dat$total05R1;

# 2005 A vote

m05 <- multinomRob(list(
  Ahmadinejad.A ~ larijani + ghalibaf + moeen + karobi+ ahmadi_nejad + rafsanjani,
  Rafsanjani.A ~ 0),
  dat, MLEonly=TRUE);
summary(m05);
m05 <- multinomRob(list(
  Ahmadinejad.A ~ larijani + ghalibaf + moeen + karobi+ ahmadi_nejad + rafsanjani,
  Rafsanjani.A ~ 0),
  dat);
summary(m05);

# outliers from constant-vote-proportion model
linpred <- exp(cbind(1,
  as.matrix(dat[,c("larijani","ghalibaf","moeen","karobi","ahmadi_nejad","rafsanjani")])) %*%
  m05$coefficients);
pmat <- linpred/apply(linpred,1,sum);
rmat <- as.matrix(dat[,c("Ahmadinejad.A","Rafsanjani.A")])/
    dat$total05-pmat;
widx <- apply(m05$weights[,-1,drop=FALSE]==0,1,any);
usermat <- rmat;
usermat[m05$weights[,-1,drop=FALSE]!=0] <- NA;
cbind(dat$obsnum[widx], usermat[widx,]);

# downweighted obs from constant-vote-proportion model
widx <- apply(m05$weights[,-1,drop=FALSE]<1,1,any);
usermat <- rmat;
usermat[m05$weights[,-1,drop=FALSE]==1] <- NA;
cbind(dat$obsnum[widx], usermat[widx,]);

# 2005 A vote by logits

dat[,
  c("Llarijani","Lghalibaf","Lmoeen","Lkarobi","Lahmadi_nejad","Lrafsanjani")] <-
log(dat[,
  c("larijani","ghalibaf","moeen","karobi","ahmadi_nejad","rafsanjani")]/
  dat[,"mehr_alizadeh"]);

m05L <- multinomRob(list(
  Ahmadinejad.A ~ Llarijani + Lghalibaf + Lmoeen + Lkarobi+ Lahmadi_nejad + Lrafsanjani,
  Rafsanjani.A ~ 0),
  dat, MLEonly=TRUE);
summary(m05L);
m05L <- multinomRob(list(
  Ahmadinejad.A ~ Llarijani + Lghalibaf + Lmoeen + Lkarobi+ Lahmadi_nejad + Lrafsanjani,
  Rafsanjani.A ~ 0),
  dat);
summary(m05L);

# outliers from constant-vote-proportion model
linpred <- exp(cbind(1,
  as.matrix(dat[,c("Llarijani","Lghalibaf","Lmoeen","Lkarobi","Lahmadi_nejad","Lrafsanjani")])) %*%
  m05L$coefficients);
pmat <- linpred/apply(linpred,1,sum);
rmat <- as.matrix(dat[,c("Ahmadinejad.A","Rafsanjani.A")])/
    dat$total05-pmat;
widx <- apply(m05L$weights[,-1,drop=FALSE]==0,1,any);
usermat <- rmat;
usermat[m05L$weights[,-1,drop=FALSE]!=0] <- NA;
cbind(dat$obsnum[widx], usermat[widx,]);

# downweighted obs from constant-vote-proportion model
widx <- apply(m05L$weights[,-1,drop=FALSE]<1,1,any);
usermat <- rmat;
usermat[m05L$weights[,-1,drop=FALSE]==1] <- NA;
cbind(dat$obsnum[widx], usermat[widx,]);
